/*
 *  Copyright (C) 2007  Thadeu Lima de Souza Cascardo <cascardo@holoscopio.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */


#include <atompub/atom.h>

#include <glib.h>

struct _atom_ctx
{
  AtomError *error;
  AtomConfig *config;
  AtomBackend *backend;
  AtomFrontend *frontend;
  GHashTable *bemap;
  GHashTable *femap;
};

AtomCtx *
atom_ctx_new ()
{
  AtomCtx *ctx;
  ctx = g_slice_new (AtomCtx);
  ctx->error = NULL;
  ctx->config = NULL;
  ctx->backend = NULL;
  ctx->frontend = NULL;
  ctx->bemap = g_hash_table_new_full (g_str_hash, g_str_equal,
                                      g_free, g_free);
  ctx->femap = g_hash_table_new_full (g_str_hash, g_str_equal,
                                      g_free, g_free);
  return ctx;
}

void
atom_ctx_delete (AtomCtx *ctx)
{
  if (ctx->error)
    atom_error_delete (ctx->error);
  if (ctx->backend)
    atom_backend_delete (ctx->backend);
  if (ctx->frontend)
    atom_frontend_delete (ctx->frontend);
  if (ctx->bemap)
    g_hash_table_destroy (ctx->bemap);
  if (ctx->femap)
    g_hash_table_destroy (ctx->femap);
  g_slice_free (AtomCtx, ctx);
}

void
atom_error_set (AtomCtx *ctx, AtomError *error)
{
  if (ctx->error)
    atom_error_delete (ctx->error);
  ctx->error = error;
}

AtomError *
atom_error_get (AtomCtx *ctx)
{
  return ctx->error;
}

AtomConfig *
atom_config (AtomCtx *ctx)
{
  return ctx->config;
}

void
atom_config_set (AtomCtx *ctx, AtomConfig *config)
{
  ctx->config = config;
}

AtomBackend *
atom_backend (AtomCtx *ctx)
{
  return ctx->backend;
}

void
atom_backend_set (AtomCtx *ctx, AtomBackend *backend)
{
  ctx->backend = backend;
}

AtomFrontend *
atom_frontend (AtomCtx *ctx)
{
  return ctx->frontend;
}

void
atom_frontend_set (AtomCtx *ctx, AtomFrontend *frontend)
{
  ctx->frontend = frontend;
}

void
atom_map_backend_requests (AtomCtx *ctx, char **reqs,
                           AtomEntry **entries, size_t len)
{
  int i;
  for (i = 0; i < len; i++)
    {
      char *key = g_strdup (atom_entry_id (entries[i]));
      char *val = g_strdup (reqs[i]);
      g_hash_table_replace (ctx->bemap, key, val);
    }
}

void
atom_map_frontend_requests (AtomCtx *ctx, char **reqs,
                            AtomEntry **entries, size_t len)
{
  int i;
  for (i = 0; i < len; i++)
    {
      char *key = g_strdup (reqs[i]);
      char *val = g_strdup (atom_entry_id (entries[i]));
      g_hash_table_replace (ctx->femap, key, val);
    }
}

AtomID *
atom_id_new_from_frontend (AtomCtx *ctx, char * req)
{
  char *id;
  if (g_hash_table_lookup_extended (ctx->femap, req, NULL, &id))
    return atom_id_new (id);
  return NULL;
}

char *
atom_id_to_backend (AtomCtx *ctx, AtomID *id)
{
  char *key = atom_id_string (id);
  char *val;
  if (g_hash_table_lookup_extended (ctx->bemap, key, NULL, &val))
    return val;
  return NULL;
}
